
local addon = KiwiPlates

local UnitAura = UnitAura
local UnitIsUnit = UnitIsUnit
local UnitReaction = UnitReaction
local CooldownFrame_Set = CooldownFrame_Set
local C_GetNamePlateForUnit = C_NamePlate.GetNamePlateForUnit

addon.defaults.auras = {}

local buffsGlobalList  
local ORIGINAL   = {}
local emptyTable = {}
local emptyFunc  = function() end

local function UNIT_AURA(self, unit)

	local plateFrame = C_GetNamePlateForUnit(unit)
	if not plateFrame then return end
	local UnitFrame = plateFrame.UnitFrame
	if not UnitFrame then return end
	local db         = UnitFrame.__skin
	local buffFrame  = UnitFrame.BuffFrame
	local buffList   = buffFrame.buffList
	local buffIndex  = 1
	local filter     = (UnitIsUnit("player", unit) and "HELPFUL|INCLUDE_NAME_PLATE_ONLY") or ((UnitReaction("player", unit) or 5)<5 and "HARMFUL|PLAYER|INCLUDE_NAME_PLATE_ONLY") or "NONE"
	buffFrame.unit   = unit
	buffFrame.filter = filter
	if filter ~= "NONE" then
		local buffsNames  = db.buffsNames or buffsGlobalList
		local isBlackList = db.buffsIsBlackList
		for i = 1, 32 do
			local name, _, texture, count, _, duration, expiration = UnitAura(unit, i, filter)
			if not name then break end
			if isBlackList == not buffsNames[name] then
				local buff = buffList[buffIndex]
				if not buff then
					local w, h, s = db.buffsWidth  or 20, db.buffsHeight or 14, db.buffsSpacing or 4
					buff = CreateFrame("Frame", UnitFrame:GetName() .. "Buff" .. buffIndex, buffFrame, "NameplateBuffButtonTemplate")
					buff.layoutIndex = buffIndex
					buff:SetSize( w, h )
					buff:SetPoint( 'TOPLEFT', buffFrame, 'TOPLEFT', (buffIndex-1) * (w+s), 0 )
					buff.Icon:SetSize( w-2, h-2 )
					buff.Icon:SetTexCoord( 0.1, 0.9, 0.1, 0.9 )
					buff:SetMouseClickEnabled(false)						
					buffList[buffIndex] = buff
				end
				buff:SetID(i)
				buff.name = name
				buff.Icon:SetTexture(texture)
				if count > 1 then
					buff.CountFrame.Count:SetText(count)
					buff.CountFrame.Count:Show()
				else
					buff.CountFrame.Count:Hide()
				end
				CooldownFrame_Set(buff.Cooldown, expiration-duration, duration, duration>0, true)
				buff:Show()
				buffIndex = buffIndex + 1
			end
		end
	end	
	-- Hide unused buffs and remember visible buffs count
	for i = buffList.__buffMax or 0, buffIndex, -1 do
		buffList[i]:Hide()
	end
	buffList.__buffMax = buffIndex - 1
	
end

function addon:LayoutAuras(UnitFrame, db)

	local buffFrame = UnitFrame.BuffFrame
	local buffList  = buffFrame.buffList
	local buffMax   = #buffList
	if buffMax>0 then
		local w, h, s = db.buffsWidth  or 20, db.buffsHeight or 14, db.buffsSpacing or 4
		local x = w+s
		for buffIndex = buffMax, 1, -1 do
			local buff = buffList[buffIndex]
			buff:SetSize( w, h )
			buff:SetPoint( 'TOPLEFT', buffFrame, 'TOPLEFT', (buffIndex-1) * x, 0 )
			buff.Icon:SetSize( w-2, h-2 )
			buff.Icon:SetTexCoord( 0.1, 0.9, 0.1, 0.9 )
		end
	end
	buffFrame:SetPoint("BOTTOM", UnitFrame.healthBar, "TOP", db.buffsOffsetX or 0, db.buffsOffsetY or 0)	

end

--=============================================================================

local auras = addon:NewModule('auras')

function auras:Update()
	local NBCM = _G['NameplateBuffContainerMixin']
	local NPDF = _G['NamePlateDriverFrame']
	if addon.__userDefinedPlates and addon.db.auras.enabled~=nil then
		addon:RegisterEvent("UNIT_AURA")
		addon.UNIT_AURA = UNIT_AURA
		if not next(ORIGINAL) then
			ORIGINAL.OnUnitAuraUpdate = NPDF.OnUnitAuraUpdate
			ORIGINAL.UpdateAnchor     = NBCM.UpdateAnchor		
		end
		NPDF.OnUnitAuraUpdate = emptyFunc
		NBCM.UpdateAnchor     = emptyFunc
		buffsGlobalList       = addon.db.auras.buffsNames or emptyTable
	elseif next(ORIGINAL) then
		addon:UnregisterEvent("UNIT_AURA")
		addon.UNIT_AURA = emptyFunc
		NPDF.OnUnitAuraUpdate = ORIGINAL.OnUnitAuraUpdate
		NBCM.UpdateAnchor     = ORIGINAL.UpdateAnchor
		wipe(ORIGINAL)
	end
end

--=============================================================================

addon:SetupOptions( 'Skins/Settings', 'Auras', {
	buffsEnabled = {
		type = 'select',
		order = 0.1,
		width = 'normal',
		name = 'Auras Behaviour',
		get = function() 
			local v = addon.db.auras.enabled
			return (v==nil and 1) or (v and 2) or 3
		end,
		set = function(info, v) 
			if v==1 then
				addon.db.auras.enabled= nil
			else
				addon.db.auras.enabled = ( v==2 )
			end
			addon:Update(true)
		end,
		values = { 'Blizzard Default', 'User Defined', 'Hide Auras' },
	},
	buffsSpacing =  {
		type = 'range', order = 0.2, name = 'Auras Spacing', min = 0, softMax = 32, step = 1,
		get = function() return addon:GetSkin().buffsSpacing or 4 end,
		set = function(info,value)
			addon:GetSkin().buffsSpacing = value
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsOffsetX =  {
		type = 'range', order = 1, name = 'X Adjust', softMin = -64, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsOffsetX or 0 end,
		set = function(info,value)
			addon:GetSkin().buffsOffsetX = value~=0 and value or nil
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsOffsetY =  {
		type = 'range', order = 2, name = 'Y Adjust', softMin = -64, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsOffsetY or 4 end,
		set = function(info,value)
			addon:GetSkin().buffsOffsetY = value~=0 and value or nil
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},	
	buffsWidth = {
		type = 'range', order = 3, name = 'Aura Width', min = 0, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsWidth	or 20 end,
		set = function(info,value)
			addon:GetSkin().buffsWidth = value
			addon:Update(true)					
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsHeight =  {
		type = 'range', order = 4, name = 'Aura Height', min = 1, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsHeight or 14 end,
		set = function(info,value)
			addon:GetSkin().buffsHeight = value
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsBlackList = {
		type = 'select',
		order = 100,
		width = 'normal',
		name = 'Aura List Type',
		desc = 'WhiteList: Specified auras will be displayed.\nBlackList: Specified auras will be hidden.',
		get = function() 
			return addon:GetSkin().buffsIsBlackList and 1 or 2
		end,
		set = function(info, v) 
			addon:GetSkin().buffsIsBlackList = (v==1) or nil
			addon:Update(true)
		end,
		values = { 'BlackList', 'WhiteList' },
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsPrivateList = {
		type = "toggle",
		order = 101, width = "normal",
		name = "Use Private List",
		desc = "Use a private aura list specific for this skin instead of the global aura list.",
		get = function() return addon:GetSkin().buffsNamesStr~=nil end,
		set = function (_, value) 
			addon:GetSkin().buffsNamesStr = value and "" or nil
			addon:GetSkin().buffsNames    = value and {} or nil
		end,
		disabled = function() return not addon.db.auras.enabled end,				
		confirm = function(_, value)
			return (not value) and "Current aura list will be erased and the global list will be used. Are you Sure ?" or nil
		end
	},
	buffsNames = {
		type = 'input',
		order = 102,
		name = 'Aura List',
		multiline = 7,
		width = 'full',
		get = function(info)
			return addon:GetSkin().buffsNamesStr or addon.db.auras.buffsNamesStr
		end,
		set = function(info, value)
			local db =  addon:GetSkin().buffsNamesStr and addon:GetSkin() or addon.db.auras
			local names = db.buffsNames or {}
			wipe(names)
			local list  = { strsplit("\n", value) }
			for i=1,#list do
				names[ strtrim(list[i]) ] = true
			end			
			db.buffsNames    = names
			db.buffsNamesStr = value
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
} )
